# ==== scan_RING.py : 
# enigma, search the genuine RING setting from the indicator

import sys
import enigmaM3
import string	
from bib_util import *
import getopt
import random
import time
import signal, os

ALPHA=string.ascii_uppercase
# ===================================
def dechiffre(msg, e ):
    crypto = ""
    for car in msg:
        crypto += e.encode( car )
    return crypto
# ===================================
def affi_score( valeur, lg):
    if valeur > 1:
        valeur = valeur / lg
    ch = ("%6.4f" % valeur)
    return ch
# ===================================
def affiche( tb ):
    ch = ""
    for x in range( len(tb) ):
        ch += ALPHA[ tb[x] ]
    return ch
# ===================================
def extrait( debut, fin ):
    ch_extrait = ALPHA[ ALPHA.find(debut) : ALPHA.find(fin) + 1 ]
    return ch_extrait
# ===================================
def decode_grund( ch ):
    tb = ch.split(":")
    if len(tb) == 1:
        return (ch[0],ch[1],ch[2])
    gr_l = extrait( tb[0][0], tb[1][0])
    gr_m = extrait( tb[0][1], tb[1][1])
    gr_r = extrait( tb[0][2], tb[1][2])
    return (gr_l, gr_m, gr_r)
# ===================================
def decode_ring( ch ):
    tb = ch.split(":")
    if len(tb) == 1:
        return (ch[0], ch[1], ch[2])
    r_l = extrait( tb[0][0], tb[1][0])
    r_m = extrait( tb[0][1], tb[1][1])
    r_r = extrait( tb[0][2], tb[1][2])
    return (r_l, r_m, r_r)
# ===================================
def sig_handler(signum, frame):
    global last_time
    if time.time() - last_time < 10: sys.exit(0)
    print("[",WALZENLAGE,RING,GRUND,int(time.time()-start_time)/60,end=' ')
    print(nb_dechif,"]")
    last_time = time.time()
# ===================================
def usage():
    print("usage: find_ring.py [option...] ")
    print("  -h             This help")
    print("  -v             Verbose (mode Debug)")
    print("  -c crypto      The cryptogram (a phrase)")
    print("  -p plain       The plain (a phrase)")
    print("  -G XYZ         The Grund used to cipher the message key")
    print("  -i indicator   The indicator (the message key ciphered)")
    print("  -U UKW         The Reflector, by default B")
    print("  -W XYZ         The Walzenlage. Example: -W I,II,III")
    print("  -S steckers    The Steckers, ex: AH:JQ:LP:EZ:RT:NF, nothing by default")
    print("Ctrl-C prints the progression, two Ctrl-C stops the program")
# ------------------ DEBUT PROGRAMME ---------------------
# ----- on traite les options
try:
    opts, args = getopt.getopt(sys.argv[1:],
        "hvc:p:G:i:U:W:s:", ["help", "verbose", "crypto=", "plain=", 
        "grund=", "indi=", "ukw=", "walzenlage=", "steckers=", ])
except getopt.GetoptError as err:
    usage()
    sys.exit(2)

DEBUG=False
msg = ""
GRUND = ""
RING_UKW  = string.ascii_uppercase
RING_LEFT = string.ascii_uppercase
RING_MIDDLE = string.ascii_uppercase
RING_RIGHT  = string.ascii_uppercase
UKW = 'B' 
STECKERS = ""
CIPHER = ""
PLAIN = ""
INDICATOR = ""

for o, a in opts:
    if o == "-v":
        DEBUG = True
    elif o in ("-h", "--help"):
        usage()
        sys.exit()
    elif o in ("-c", "--crypto"):
        CIPHER = a.upper()
    elif o in ("-p", "--plain"):
        PLAIN = a.upper()
    elif o in ("-i", "--indi"):
        INDICATOR = a
    elif o in ("-W", "--walzenlage"):
        WALZENLAGE = a
    elif o in ("-G", "--grund"):
        GRUND = a
    elif o in ("-U", "--ukw"):
        UKW = a
    elif o in ("-s", "--steckers"):
        STECKERS = a
    else:
        assert False, "unhandled option"
if CIPHER == "":
    usage()
    sys.exit(1)
# ---------------------------
start_time = time.time() ; last_time = 0
signal.signal(signal.SIGINT, sig_handler)

tb = WALZENLAGE.split(",")
LEFT = tb[0]; MIDDLE = tb[1]; RIGHT = tb[2] 
RING='AAA'; 
c1 = enigmaM3.Enigma(UKW,LEFT,MIDDLE,RIGHT,STECKERS,RING,GRUND,DEBUG)
c2 = enigmaM3.Enigma(UKW,LEFT,MIDDLE,RIGHT,STECKERS,RING,GRUND,DEBUG)
for r_l in RING_LEFT:
     for r_m in RING_MIDDLE:
      for r_r in RING_RIGHT:
         RING =  r_l + r_m + r_r
         c1.setRing( RING )
         c1.setGrund( GRUND )
         true_grund = dechiffre( INDICATOR, c1 )
         c2.setRing( RING )
         c2.setGrund( true_grund ) 
         plain = dechiffre( CIPHER, c2 )
         if plain == PLAIN:
            print("Ringstellung and Spruchschlussel: ", RING, true_grund)
            sys.exit(1)
       

